<?php

declare(strict_types=1);

namespace hood\vk\api;

use Exception;
use hood\vk\robots\robot;
use ArrayAccess;

/**
 * Вложения
 */
class data implements ArrayAccess
{
    /**
     * @var array $data Вложения
     */
    protected array $data = [];

    /**
     * Конструктор
     * 
     * @var robot $robot Робот
     */
    public function __construct(
        protected robot $robot
    ) {
    }

    /**
     * Записать вложения
     * 
     * @param string|array $data Вложения
     * 
     * @return self
     */
    public function addData(string ...$data): self
    {
        // Если вложений больше 10
        if (count($this->data) + count($data) > 10) {
            throw new Exception('Превышен лимит вложений (10)');
        }

        // Запись вложений
        $this->data = array_merge($this->data, $data);

        return $this;
    }

    /**
     * Очистить вложения
     */
    public function clear(): self
    {
        // Очистка вложений
        $this->data = [];

        return $this;
    }

    /**
     * Прочитать свойство
     *
     * @param string $name Название
     * 
     * @return mixed
     */
    public function __get(string $name): mixed
    {
        return match ($name) {
            'data' => $this->data,
            default => throw new Exception('Свойство не найдено: ' . $name)
        };
    }

    /**
     * Записать по смещению
     */
    public function offsetSet(mixed $offset = null, ...$value): void
    {
        if ($offset == null) {
            $this->addData($value);
        }
    }

    /**
     * Прочитать по смещению
     */
    public function offsetGet(mixed $offset = null): mixed
    {
        if ($offset == 0) {
            return $this;
        }    
    }

    /**
     * Проверка существования смещения
     */
    public function offsetExists(mixed $offset): bool
    {
        if ($offset == 'data') {
            return empty($this->data);
        }
    }

    /**
     * Удалить по смещению
     */
    public function offsetUnset(mixed $offset = null): void
    {
        if ($offset == null) {
            $this->settings = [];
        }
    }
}
