<?php

declare(strict_types=1);

namespace hood\vk\api\methods;

use Exception;

use hood\accounts\vk;
use hood\vk\robots\robot;
use hood\vk\api\data;

/**
 * Сообщение
 *
 * @method public static function put(RobotAbstract $from, int $to, string $message, int $mode = 2) Отправить сообщение
 *
 * @see https://vk.com/dev/messages.send
 * @see https://vk.com/dev/messages.getById
 *
 * @package hood\vk\api
 * @author Arsen Mirzaev Tatyano-Muradovich <red@hood.su>
 *
 * @todo Доработать строгий режим отправки: проверку сообщения в беседе (не имеет ID сообщений)
 */
final class messages extends method
{
    /**
     * @param int $mode Режим отправки
     */
    protected int $mode = 1;

    /**
     * @param string|null $text Текст
     */

    /**
     *  @var data Вложения
     */
    protected data $data;

    /**
     * Создать сообщение
     *
     * @param robot $robot Робот
     */
    public function __construct(
        protected robot $robot,
        protected string|null $text = null
    ) {
    }

    /**
     * Записать текст
     *
     * @param string $text Текст
     *
     * @return self
     */
    public function text(string $text): self
    {
        // Записать текст
        if (!isset($this->text)) {
            $this->text = $text;
        } else {
            $this->text .= $text;
        }

        return $this;
    }

    /**
     * Записать фото
     *
     * @param $img Фото
     *
     * @return self
     */
    public function image(...$pathPhoto): self
    {
        // Перебор фото
        foreach ($pathPhoto as $photo) {

            // Загрузить фото
            $id = $this->robot->photo()->getPhoto($photo);

            // Записать к вложениям
            $this->attachments($id);
        }

        return $this;
    }

    /**
     * Записать вложение
     *
     * @param $attachments Вложения
     */
    public function attachments(string ...$attachments): self
    {
        if (isset($this->data)) {
            // Если вложения инициализированны

            // Записать вложение
            $this->data->addData(...$attachments);
        } else {
            // Если вложения не инициализированны

            // Инициализация вложений
            $this->data = new data($this->robot);

            // Записать вложение
            $this->data->addData(...$attachments);
        }

        return $this;
    }

    /**
     * Отправить сообщение
     *
     * @param int|string|array $destination Получатель
     *
     * @see https://vk.com/dev/messages.send
     *
     * @return array Ответ сервера
     */
    public function send(int|string|array $destination): mixed
    {
        // Идентификатор
        $random_id = time();

        if ($this->mode = 1) {
            // Перемножение (по умолчанию)
            $random_id *= rand();
        }

        // Реиницилазиция
        $this->robot->api->reinit();

        // Цель отправки
        $this->robot->api->chooseDestination($destination);

        // Идентификатор сообщения
        $this->robot->api->random_id = $random_id;

        // Текст
        $this->robot->api->message = $this->text;

        // Фильтрация вложений, если они инициализированны
        if (isset($this->data)) {

            /* $forward_messages = [];
            $attachments = $this->attachment->attachments; //Indirect modification of overloaded property hood\vk\api\attachments::$attachments has no effect
            foreach ($attachments as &$attachment) {
                if (iconv_substr($attachment, 0,  7, "UTF-8") === 'message') {

                    // Если среди вложений найдено сообщение для пересылки
                    $forward_messages[] = $attachment;
                    unset($attachment);
                }
            } */

            if (!empty($forward_messages)) {
                // Если есть пересылаемые сообщения
                $this->robot->api['forward_messages'] = implode(',', $forward_messages);
            }

            //if (!empty($this->attachment->attachments)) {
                // Если есть вложения
                //throw new Exception('СУКАААА');
                $this->robot->api['attachment'] = implode(',', $this->data->data);
            //}
        }

        // Запрос
        $request = $this->robot->browser->request('POST', 'messages.send', ['form_params' => $this->robot->api->settings]);

        // Очистка
        //unset($settings);

        if ($this->mode >= 2) {
            // Если установлен режим 2 (усиленная проверка отправленного сообщения)

            if (!empty($request["response"])) {
                // Ответ получен

                // Реиницилазиция
                $this->robot->api->reinit();

                // Запрашиваемые сообщения
                $this->robot->api['message_ids'] = $request["response"];

                // Запрос
                if ($this->robot->browser->request('POST', 'messages.getById', ['form_params' => $this->robot->api->settings])['response']['count'] === 0) {
                    // Сообщения не существует

                    // Повторная отправка
                    $this->send($destination);
                }
            } else {
                // Что-то придумать :)
            }
        }

        return $request->getBody();
    }

    /**
     * Записать свойство
     *
     * @param string $name Название
     * @param mixed $value Значение
     *
     * @return void
     */
    public function __set(string $name, mixed $value): void
    {
        match ($name) {
            'data' => !isset($this->data) ? $this->data[0]->addData($value) : throw new Exception('Вложения не инициализированны'),
            default => throw new Exception('Свойство не найдено: ' . $name)
        };
    }

    /**
     * Прочитать свойство
     *
     * @param string $name Название
     *
     * @return mixed
     */
    public function __get(string $name): mixed
    {
        return match ($name) {
            'text' => $this->text ?? throw new Exception('Текст не инициализирован'),
            'data' => isset($this->data) ? $this->data[0]->data : throw new Exception('Вложения не инициализированны'),
            default => throw new Exception('Свойство не найдено: ' . $name)
        };
    }

    /**
     * Получить информацию о сообщении
     *
     * @return array Информация о сообщении
     */
    public function info()/* : array */
    {
        //Инициализация
        //$settings = $this->init();
        $this->robot->api->init();

        // Цель отправки
        $this->robot->api->chooseDestination($this->destination);

        // Сообщение
        $this->robot->api['message'] = $this->message;

        // Режим отправки
        $this->robot->api['mode'] = $this->mode;

        // Фильтрация вложений
        $forward_messages = [];
        foreach ($this->attachments as &$attachment) {
            if (iconv_substr(str: $attachment, offset: 0, length: 7, charset: "UTF-8") === 'message') {
                // Если среди вложений найдено сообщение для пересылки

                $forward_messages[] = $attachment;
                unset($attachment);
            }
        }

        if (!empty($forward_messages)) {
            // Если есть пересылаемые сообщения

            $this->robot->api['forward_messages'] = implode(',', $forward_messages);
        }

        if (!empty($this->attachments)) {
            // Если есть вложения

            $this->robot->api['attachment'] = implode(',', $this->attachments);
        }
    }
}
